  --
  -- AbbeyST
  -- Class for all sprayers
  --
  -- @author  Stefan Geiger
  -- @date  24/02/08
  --
  -- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
  
  source("dataS/scripts/vehicles/specializations/SetTurnedOnEvent.lua");
  source("dataS/scripts/vehicles/specializations/SprayerAreaEvent.lua");
  source("dataS/scripts/vehicles/specializations/SprayerSetIsFillingEvent.lua");
  AbbeyST = {};
  
  
  AbbeyST.SPRAYTYPE_UNKNOWN = 0;
  AbbeyST.NUM_SPRAYTYPES = 0;
  
  AbbeyST.sprayTypes = {};
  AbbeyST.sprayTypeIndexToDesc = {};
  
  AbbeyST.sprayTypeToFillType = {};
  AbbeyST.fillTypeToSprayType = {};
  
function AbbeyST.registerSprayType(name, pricePerLiter, litersPerSqmPerSecond, hudOverlayFilename)
      local key = "SPRAYTYPE_"..string.upper(name);
      if AbbeyST[key] == nil then
          AbbeyST.NUM_SPRAYTYPES = AbbeyST.NUM_SPRAYTYPES+1;
          AbbeyST[key] = AbbeyST.NUM_SPRAYTYPES;
  
          local desc = {name = name, index = AbbeyST.NUM_SPRAYTYPES};
          desc.pricePerLiter = pricePerLiter;
          desc.litersPerSqmPerSecond = litersPerSqmPerSecond;
          desc.hudOverlayFilename = hudOverlayFilename;
  
 
          AbbeyST.sprayTypes[name] = desc;
          AbbeyST.sprayTypeIndexToDesc[AbbeyST.NUM_SPRAYTYPES] = desc;
  
          local fillType = Fillable.registerFillType(name)
          AbbeyST.sprayTypeToFillType[AbbeyST.NUM_SPRAYTYPES] = fillType;
          AbbeyST.fillTypeToSprayType[fillType] = AbbeyST.NUM_SPRAYTYPES;
      end;
end;
  
  AbbeyST.registerSprayType("fertilizer", 0.3, 0.5, "");
  AbbeyST.registerSprayType("manure", 0.01, 0.5, "");
  AbbeyST.registerSprayType("liquidManure", 0.01, 0.5, "");
  
function AbbeyST.prerequisitesPresent(specializations)
      return SpecializationUtil.hasSpecialization(Fillable, specializations) and SpecializationUtil.hasSpecialization(PowerShaft, specializations);
end;
 
function AbbeyST:load(xmlFile)
  
      assert(self.setIsTurnedOn == nil, "AbbeyST needs to be the first specialization which implements setIsTurnedOn");
      self.setIsTurnedOn = AbbeyST.setIsTurnedOn;
      self.getIsTurnedOnAllowed = AbbeyST.getIsTurnedOnAllowed;
  
      assert(self.setIsSprayerFilling == nil, "AbbeyST needs to be the first specialization which implements setIsSprayerFilling");
      self.setIsSprayerFilling = AbbeyST.setIsSprayerFilling;
      self.addSprayerFillTrigger = AbbeyST.addSprayerFillTrigger;
      self.removeSprayerFillTrigger = AbbeyST.removeSprayerFillTrigger;
  
      self.fillLitersPerSecond = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillLitersPerSecond"), 500);
      self.isSprayerFilling = false;
  
      self.sprayLitersPerSecond = {};
      local i=0;
      while true do
          local key = string.format("vehicle.sprayUsages.sprayUsage(%d)", i);
          if not hasXMLProperty(xmlFile, key) then
              break;
          end;
          local fillType = getXMLString(xmlFile, key.. "#fillType");
          local litersPerSecond = getXMLFloat(xmlFile, key.. "#litersPerSecond");
          if fillType ~= nil and litersPerSecond ~= nil then
              local fillTypeInt = Fillable.fillTypeNameToInt[fillType];
              if fillTypeInt ~= nil then
                  self.sprayLitersPerSecond[fillTypeInt] = litersPerSecond;
                  if self.defaultSprayLitersPerSecond == nil then
                      self.defaultSprayLitersPerSecond = litersPerSecond;
                  end;
              else
                  print("Warning: Invalid spray usage fill type '"..fillType.."' in '" .. self.configFileName.. "'");
              end;
          end;
          i = i+1;
      end;
      if self.defaultSprayLitersPerSecond == nil then
          print("Warning: No spray usage specified for '" .. self.configFileName.. "'. This AbbeyST will not use any spray.");
         self.defaultSprayLitersPerSecond = 0;
      end;
  
      self.sprayValves = {};
  
      if self.isClient then
          local psFile = getXMLString(xmlFile, "vehicle.sprayParticleSystem#file");
          if psFile ~= nil then
              local i=0;
              while true do
                  local baseName = string.format("vehicle.sprayValves.sprayValve(%d)", i);
                  local node = getXMLString(xmlFile, baseName.. "#index");
                  if node == nil then
                      break;
                  end;
                  node = Utils.indexToObject(self.components, node);
                  if node ~= nil then
                      local sprayValve = {};
                      sprayValve.particleSystems = {};
                      Utils.loadParticleSystem(xmlFile, sprayValve.particleSystems, "vehicle.sprayParticleSystem", node, false, nil, self.baseDirectory);
                      table.insert(self.sprayValves, sprayValve);
                  end;
                  i = i+1;
              end;
          end;
  
          local spraySound = getXMLString(xmlFile, "vehicle.spraySound#file");
          if spraySound ~= nil and spraySound ~= "" then
              spraySound = Utils.getFilename(spraySound, self.baseDirectory);
              self.spraySound = createSample("spraySound");
              self.spraySoundEnabled = false;
              loadSample(self.spraySound, spraySound, false);
              self.spraySoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.spraySound#pitchOffset"), 1);
              self.spraySoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.spraySound#volume"), 1);
          end;
      end;
  
      self.sprayerFillTriggers = {};
      self.sprayerFillActivatable = SprayerFillActivatable:new(self);
  
      self.isTurnedOn = false;
      self.speedViolationMaxTime = 1000;
      self.speedViolationTimer = self.speedViolationMaxTime;
	  
 	self.setVehicleRpmUp = SpecializationUtil.callSpecializationsFunction("setVehicleRpmUp");
	self.saveMinRpm = 0; 
    self.printWarningTime = 0;
    self.printWarningTime1 = 0;
    self.printWarningTime2 = 0;
	  
	self.setArmOne = SpecializationUtil.callSpecializationsFunction("setArmOne");
	self.ArmOneAnimation = getXMLString(xmlFile, "vehicle.armOne#animationName");
	self.ArmOne = false;
	
      if self.isClient then
          local fillingSound = getXMLString(xmlFile, "vehicle.fillingSound#file");
          if fillingSound ~= nil and fillingSound ~= "" then
              fillingSound = Utils.getFilename(fillingSound, self.baseDirectory);
              self.fillingSound = createSample("fillingSound");
              self.fillingSoundEnabled = false;
              loadSample(self.fillingSound, fillingSound, false);
              self.fillingSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillingSound#pitchOffset"), 1);
              self.fillingSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillingSound#volume"), 1);
          end;
      end; 

	  self.emptyParticleSystems = {};
      local i = 0;
      while true do
        local namei = string.format("vehicle.emptyParticleSystems.emptyParticleSystems(%d)", i);
		local nodei = Utils.indexToObject(self.components, getXMLString(xmlFile, namei .. "#index"));
		if nodei == nil then
			break;
		end; 
        Utils.loadParticleSystem(xmlFile, self.emptyParticleSystems, namei, nodei, false, nil, self.baseDirectory)		
		i = i +1;		
      end;

	self.overlay = Utils.getFilename("textures/manure_hud.png", self.baseDirectory);
    self.FruitTypeOverlay = Overlay:new("hudPDAControl", self.overlay, g_currentMission.fruitSymbolX, g_currentMission.fruitSymbolY, g_currentMission.fruitSymbolSize, g_currentMission.fruitSymbolSize * (4 / 3)); 
end;
  
function AbbeyST:delete()
  
    g_currentMission:removeActivatableObject(self.sprayerFillActivatable);
  
    for k,sprayValve in pairs(self.sprayValves) do
        Utils.deleteParticleSystem(sprayValve.particleSystems);
    end;
  
    if self.spraySound ~= nil then
        delete(self.spraySound);
    end;
    if self.fillingSound ~= nil then
        delete(self.fillingSound);
    end;
 	Utils.deleteParticleSystem(self.emptyParticleSystems);
end;
  
function AbbeyST:readStream(streamId, connection)
    local turnedOn = streamReadBool(streamId);
    local isSprayerFilling = streamReadBool(streamId);
    self:setIsTurnedOn(turnedOn, true);
    self:setIsSprayerFilling(isSprayerFilling, true);
    self:setArmOne(streamReadBool(streamId), true);
end;
  
function AbbeyST:writeStream(streamId, connection)
    streamWriteBool(streamId, self.isTurnedOn);
    streamWriteBool(streamId, self.isSprayerFilling);
	streamWriteBool(streamId, self.ArmOne);	
end;
  
function AbbeyST:readUpdateStream(streamId, timestamp, connection)
end;
  
function AbbeyST:writeUpdateStream(streamId, connection, dirtyMask)
end;
  
function AbbeyST:mouseEvent(posX, posY, isDown, isUp, button)
end;
  
function AbbeyST:keyEvent(unicode, sym, modifier, isDown)
end;
 
function AbbeyST:update(dt)
  
    if self.isClient then
        if self:getIsActiveForInput() then
            if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) and not self.PTOId then
                  self:setIsTurnedOn(not self.isTurnedOn);
            end;
			if InputBinding.isPressed(InputBinding.IMPLEMENT_EXTRA) and self.PTOId then
				self.printWarningTime = self.time + 1000;
			end;	
			if InputBinding.isPressed(InputBinding.ACTIVATE_OBJECT) and self.PTOId then
				self.printWarningTime1 = self.time + 1000;
			end;
			if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) and not (self.isTurnedOn or self.isSprayerFilling) then
				self:setArmOne(not self.isArmOneOn);
			end;
			if (InputBinding.isPressed(InputBinding.IMPLEMENT_EXTRA) or InputBinding.isPressed(InputBinding.ACTIVATE_OBJECT)) and self.isArmOneOn then
				self.printWarningTime2 = self.time + 1000;
			end;
        end;
    end;
  
end;
  
function AbbeyST:updateTick(dt)
  
      if self:getIsActive() then  
		if not self.attacherVehicle.isMotorStarted then
			self:setIsTurnedOn(false);
            self:setIsSprayerFilling(false);
		end;
		if self.PTOId then
			self:setIsTurnedOn(false);
            self:setIsSprayerFilling(false);
		end;
		if self.isArmOneOn then
			self:setIsTurnedOn(false);
            self:setIsSprayerFilling(false);
		end;
		local isKeyEvent = false;	
		if self.isTurnedOn or self.isSprayerFilling then
 			isKeyEvent = true;	
		else
 			isKeyEvent = false;	
		end;
		self:setVehicleRpmUp(dt, isKeyEvent);
		if self.isTurnedOn then 
				  self.lastSprayingArea = 0;
				  if self:doCheckSpeedLimit() and self.lastSpeed*3600 > 31 then
					  self.speedViolationTimer = self.speedViolationTimer - dt;
				  else
					  self.speedViolationTimer = self.speedViolationMaxTime;
				  end;
	  
				  if self.isServer then
					  if self.speedViolationTimer > 0 then
						  local litersPerSecond = self.sprayLitersPerSecond[self.currentFillType];
						  if litersPerSecond == nil then
							  litersPerSecond = self.defaultSprayLitersPerSecond;
						  end
						  local usage = litersPerSecond * dt*0.001;
	  
						  local hasSpray = false;
	  
						  if self.capacity == 0 or self:getIsHired() then
							  hasSpray = true;
							  local sprayType = AbbeyST.fillTypeToSprayType[self.currentFillType];
							  if sprayType ~= nil then
								  local sprayTypeDesc = AbbeyST.sprayTypeIndexToDesc[sprayType];
								  local delta = usage*sprayTypeDesc.pricePerLiter
								  g_currentMission.missionStats.expensesTotal = g_currentMission.missionStats.expensesTotal + delta;
								  g_currentMission.missionStats.expensesSession = g_currentMission.missionStats.expensesSession + delta;
	  
								  g_currentMission:addSharedMoney(-delta);
	  
								  -- TODO update spray stats
								  --[[g_currentMission.missionStats.sprayUsageTotal = g_currentMission.missionStats.sprayUsageTotal + usage;
								  g_currentMission.missionStats.sprayUsageSession = g_currentMission.missionStats.sprayUsageSession + usage;]]
							  end;
						  else
							  if self.fillLevel > 0 then
								  hasSpray = true;
								  self:setFillLevel(self.fillLevel - usage, self.currentFillType);
							  end;
						  end;
	  
						  if hasSpray then
							  local cuttingAreasSend = {};
							  for k,cuttingArea in pairs(self.cuttingAreas) do
								  if self:getIsAreaActive(cuttingArea) then
									  local x,y,z = getWorldTranslation(cuttingArea.start);
									  local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
									  local x2,y2,z2 = getWorldTranslation(cuttingArea.height);
	  
									  local sqm = math.abs((z1-z)*(x2-x) - (x1-x)*(z2-z)); -- this is the cross product with y=0
	  
									  --Utils.updateSprayArea(x, z, x1, z1, x2, z2);
	  
									  self.lastSprayingArea = self.lastSprayingArea + sqm;
	  
									  table.insert(cuttingAreasSend, {x,z,x1,z1,x2,z2});
								  end;
							  end;
							  if (table.getn(cuttingAreasSend) > 0) then
								  SprayerAreaEvent.runLocally(cuttingAreasSend);
								  g_server:broadcastEvent(SprayerAreaEvent:new(cuttingAreasSend));
							  end;
						  end;
					  end;
				  end;
	  
				  if self.isClient then
					  if not self.spraySoundEnabled and self:getIsActiveForSound() then
						  playSample(self.spraySound, 0, self.spraySoundVolume, 0);
						  setSamplePitch(self.spraySound, self.spraySoundPitchOffset);
						  self.spraySoundEnabled = true;
					  end;
				  end;
	  
				  if self.fillLevel <= 0 and self.capacity ~= 0 then
					  self:setIsTurnedOn(false, true);
				  end;
			  else
				  self.speedViolationTimer = self.speedViolationMaxTime;
			end;
			if self.isTurnedOn and self.fillLevel <= 500 then
				Utils.setEmittingState(self.emptyParticleSystems, true);
			else
				Utils.setEmittingState(self.emptyParticleSystems, false);
			end;
    end;
	if self.isSprayerFilling and self.isServer then
	  local disableFilling = false;
			  if self:allowFillType(self.sprayerFillingFillType, false) then
				  local oldFillLevel = self.fillLevel;
	  
				  local delta = self.fillLitersPerSecond*dt*0.001;
	  
				  local silo = g_currentMission:getSiloAmount(self.sprayerFillingFillType);
				  if self.sprayerFillingIsSiloTrigger then
					  if silo <= 0 then
						 disableFilling = true;
					  end;
					  delta = math.min(delta, silo);
				  end;
	  
				  self:setFillLevel(self.fillLevel + delta, self.sprayerFillingFillType, true);
				  local delta = self.fillLevel - oldFillLevel;
				  if delta > 0 then
					  if self.sprayerFillingIsSiloTrigger then
						  g_currentMission:setSiloAmount(self.sprayerFillingFillType, silo - delta);
					  else
						  local sprayType = AbbeyST.fillTypeToSprayType[self.sprayerFillingFillType];
						  if sprayType ~= nil then
							  local sprayTypeDesc = AbbeyST.sprayTypeIndexToDesc[sprayType]
	  
							  local price = delta*sprayTypeDesc.pricePerLiter;
							  g_currentMission.missionStats.expensesTotal = g_currentMission.missionStats.expensesTotal + price;
							  g_currentMission.missionStats.expensesSession = g_currentMission.missionStats.expensesSession + price;
	  
							  g_currentMission:addSharedMoney(-price);
						  end;
					  end;
				  elseif self.fillLevel == self.capacity then
					  disableFilling = true;
				  end;
			  else
				  disableFilling = true;
			  end;
			if disableFilling then
					self:setIsSprayerFilling(false);
			end;

	end;
    if self.isClient then		
		if self.isSprayerFilling then
			if not self.fillingSoundEnabled and self:getIsActiveForSound() then
                playSample(self.fillingSound, 0, self.fillingSoundVolume, 0);
                setSamplePitch(self.fillingSound, self.fillingSoundPitchOffset);
                self.fillingSoundEnabled = true;
            end;
        end;
		if not self.isSprayerFilling then
			stopSample(self.fillingSound);
			self.fillingSoundEnabled = false;
		end;			
	end;
end;
  
function AbbeyST:draw()
  
    if self.isClient then
        if self.fillLevel <= 0 and self.capacity ~= 0 then
            g_currentMission:addExtraPrintText(g_i18n:getText("FirstFillTheTool"));
        end;
        if not self.PTOId then 
			if self.isTurnedOn then
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
			else
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
			end;
		end;
  
        if math.abs(self.speedViolationTimer - self.speedViolationMaxTime) > 2 then
              g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getKeyNamesOfDigitalAction(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
        end;

		if self.printWarningTime > self.time then
			g_currentMission:addWarning(g_i18n:getText("turnON_Error"), 0.018, 0.033);
		end;
		if self.printWarningTime1 > self.time then
			g_currentMission:addWarning(g_i18n:getText("fill_Error"), 0.018, 0.033);
		end;
		if self.printWarningTime2 > self.time then
			g_currentMission:addWarning(g_i18n:getText("hatch_Error"), 0.018, 0.033);
		end;
		if not (self.isTurnedOn or self.isSprayerFilling) then
			if self.isArmOneOn then
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("close"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA2);
			else
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("open"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA2);
			end;
		end;
		if self.fillLevel > 0 then
			self.FruitTypeOverlay:render();
		end;
	end;
end;
 
function AbbeyST:onAttach(attacherVehicle)
	self.attacherVehicle = attacherVehicle;
	if self.attacherVehicleCopy == nil then
		self.attacherVehicleCopy = self.attacherVehicle;
	end;
	self.saveMinRpm = self.attacherVehicle.motor.minRpm;
end;
 
function AbbeyST:onDetach()
	for k, steerable in pairs(g_currentMission.steerables) do
		if self.attacherVehicleCopy == steerable then
			steerable.motor.minRpm = self.saveMinRpm;
			self.attacherVehicleCopy = nil;
		end;
	end;
	if self.emptyParticleSystems ~= nil then
		Utils.setEmittingState(self.emptyParticleSystems, false);
	end; 
      if self.deactivateOnDetach then
          AbbeyST.onDeactivate(self);
      else
         AbbeyST.onDeactivateSounds(self);
      end;
end;
  
function AbbeyST:onLeave()
      if self.deactivateOnLeave then
          AbbeyST.onDeactivate(self);
      else
          AbbeyST.onDeactivateSounds(self);
      end;
	if self.emptyParticleSystems ~= nil then
		Utils.setEmittingState(self.emptyParticleSystems, false);
	end; 
end;
  
function AbbeyST:onDeactivate()
      self.speedViolationTimer = self.speedViolationMaxTime;
      self:setIsTurnedOn(false, true)
      AbbeyST.onDeactivateSounds(self);
 	if self.emptyParticleSystems ~= nil then
		Utils.setEmittingState(self.emptyParticleSystems, false);
	end; 
end;
  
function AbbeyST:onDeactivateSounds()
      if self.spraySoundEnabled then
          stopSample(self.spraySound);
          self.spraySoundEnabled = false;
      end;
      if self.fillingSoundEnabled then
          stopSample(self.fillingSound);
          self.fillingSoundEnabled = false;
      end;
end;
  
function AbbeyST:getIsTurnedOnAllowed(isTurnedOn)
      if not isTurnedOn or self.fillLevel > 0 or self.capacity == 0 then
          return true;
      end;
end;
  
function AbbeyST:setIsTurnedOn(isTurnedOn, noEventSend)
      if isTurnedOn ~= self.isTurnedOn then
          if self:getIsTurnedOnAllowed(isTurnedOn) then
              SetTurnedOnEvent.sendEvent(self, isTurnedOn, noEventSend)
  
              self.isTurnedOn = isTurnedOn;
              if self.isClient then
                  for k,sprayValve in pairs(self.sprayValves) do
                      Utils.setEmittingState(sprayValve.particleSystems, self.isTurnedOn);
                  end;
  
                  if not self.isTurnedOn and self.spraySoundEnabled then
                      stopSample(self.spraySound);
                      self.spraySoundEnabled = false;
                  end;
              end;
              self.speedViolationTimer = self.speedViolationMaxTime;
          end;
      end;
end;
  
function AbbeyST:setIsSprayerFilling(isFilling, fillType, isSiloTrigger, noEventSend)
      SprayerSetIsFillingEvent.sendEvent(self, isFilling, fillType, isSiloTrigger, noEventSend)
      if self.isSprayerFilling ~= isFilling then
          self.isSprayerFilling = isFilling;
          self.sprayerFillingFillType = fillType;
          self.sprayerFillingIsSiloTrigger = isSiloTrigger;
      end;
end;
  
function AbbeyST:addSprayerFillTrigger(trigger)
      if table.getn(self.sprayerFillTriggers) == 0 then
          g_currentMission:addActivatableObject(self.sprayerFillActivatable);
      end;
      table.insert(self.sprayerFillTriggers, trigger);
end;
  
function AbbeyST:removeSprayerFillTrigger(trigger)
      for i=1, table.getn(self.sprayerFillTriggers) do
          if self.sprayerFillTriggers[i] == trigger then
              table.remove(self.sprayerFillTriggers, i);
              break;
          end;
      end;
      if table.getn(self.sprayerFillTriggers) == 0 then
          if self.isServer then
              self:setIsSprayerFilling(false);
          end;
          g_currentMission:removeActivatableObject(self.sprayerFillActivatable);
      end;
end;

function AbbeyST:setVehicleRpmUp(dt, isActive)
	if self.attacherVehicle ~= nil and self.saveMinRpm ~= 0 then
		if dt ~= nil then
			if isActive == true then
				self.attacherVehicle.motor.minRpm = math.max(self.attacherVehicle.motor.minRpm-dt, -1000);
			else
				self.attacherVehicle.motor.minRpm = math.min(self.attacherVehicle.motor.minRpm+dt*2, self.saveMinRpm);
			end;
		else
			self.attacherVehicle.motor.minRpm = self.saveMinRpm;
		end;
		if self.attacherVehicle.isMotorStarted then
			local fuelUsed = 0.0000011*math.abs(self.attacherVehicle.motor.minRpm);
			self.attacherVehicle:setFuelFillLevel(self.attacherVehicle.fuelFillLevel-fuelUsed);
			g_currentMission.missionStats.fuelUsageTotal = g_currentMission.missionStats.fuelUsageTotal + fuelUsed;
			g_currentMission.missionStats.fuelUsageSession = g_currentMission.missionStats.fuelUsageSession + fuelUsed;
		end;
	end;
end;

function AbbeyST:setArmOne(isArmOne,noEventSend)
	SetArmOneEvent.sendEvent(self, isArmOne, noEventSend);
	-- Play ArmOne animation --
	self.isArmOneOn = isArmOne;
	if self.isArmOneOn then
		if self.ArmOneAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.ArmOneAnimation, 1, nil, true);
			self.ArmOne = true;
		end;
	else
		if self.ArmOneAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.ArmOneAnimation, -1, nil, true);
			self.ArmOne = false;
		end;
	end;	
end;
  
SprayerFillActivatable = {}
local SprayerFillActivatable_mt = Class(SprayerFillActivatable);
  
function SprayerFillActivatable:new(AbbeyST)
      local self = {};
      setmetatable(self, SprayerFillActivatable_mt);
  
      self.AbbeyST = AbbeyST;
      self.activateText = "unknown";
  
      self.currentTrigger = nil;
  
      return self;
end;
  
  
function SprayerFillActivatable:getIsActivatable()
    self.currentTrigger = nil;
		if not self.AbbeyST:getIsActiveForInput() or self.AbbeyST.fillLevel == self.AbbeyST.capacity then
			  return false;
		end;
      -- find the first trigger which is activable
      for i=1, table.getn(self.AbbeyST.sprayerFillTriggers) do
          local trigger = self.AbbeyST.sprayerFillTriggers[i];
          if trigger:getIsActivatable(self.AbbeyST) then
              self.currentTrigger = trigger;
              self:updateActivateText();
              return true;
          end;
      end;
      return false;
end;
  
function SprayerFillActivatable:onActivateObject()
      self.AbbeyST:setIsSprayerFilling(not self.AbbeyST.isSprayerFilling, self.currentTrigger.fillType, self.currentTrigger.isSiloTrigger);  
	  self:updateActivateText();
      g_currentMission:addActivatableObject(self);
end;
  
function SprayerFillActivatable:drawActivate()
      -- TODO draw icon
end;
  
function SprayerFillActivatable:updateActivateText()
      if self.AbbeyST.isSprayerFilling then
          self.activateText = string.format(g_i18n:getText("stop_refill_OBJECT"), self.AbbeyST.typeDesc);
      else
          self.activateText = string.format(g_i18n:getText("refill_OBJECT"), self.AbbeyST.typeDesc);
     end;
end;

SetArmOneEvent = {};
SetArmOneEvent_mt = Class(SetArmOneEvent, Event);

InitEventClass(SetArmOneEvent, "SetArmOneEvent");

function SetArmOneEvent:emptyNew()
    local self = Event:new(SetArmOneEvent_mt);
    self.className="SetArmOneEvent";
    return self;
end;

function SetArmOneEvent:new(vehicle, isArmOne)
    local self = SetArmOneEvent:emptyNew()
    self.vehicle = vehicle;
	self.isArmOne = isArmOne;
    return self;
end;

function SetArmOneEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.isArmOne = streamReadBool(streamId);
    self.vehicle = networkGetObject(id);
    self:run(connection);
end;

function SetArmOneEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteBool(streamId, self.isArmOne);
end;

function SetArmOneEvent:run(connection)   
	self.vehicle:setArmOne(self.isArmOne, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(SetArmOneEvent:new(self.vehicle, self.isArmOne), nil, connection, self.vehicle);
    end;
end;

function SetArmOneEvent.sendEvent(vehicle, isArmOne, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetArmOneEvent:new(vehicle, isArmOne), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetArmOneEvent:new(vehicle, isArmOne));
		end;
	end;
end;

